@@ -24,16 +24,12 @@ import (
 	"strings"
 	"time"
 
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/client-go/kubernetes"
-
 	"github.com/GoogleContainerTools/skaffold/v2/pkg/diag"
 	"github.com/GoogleContainerTools/skaffold/v2/pkg/diag/validator"
 	sErrors "github.com/GoogleContainerTools/skaffold/v2/pkg/skaffold/errors"
 	"github.com/GoogleContainerTools/skaffold/v2/pkg/skaffold/event"
 	eventV2 "github.com/GoogleContainerTools/skaffold/v2/pkg/skaffold/event/v2"
 	"github.com/GoogleContainerTools/skaffold/v2/pkg/skaffold/kubectl"
-	kubernetesclient "github.com/GoogleContainerTools/skaffold/v2/pkg/skaffold/kubernetes/client"
 	"github.com/GoogleContainerTools/skaffold/v2/pkg/skaffold/output/log"
 	"github.com/GoogleContainerTools/skaffold/v2/proto/v1"
 	protoV2 "github.com/GoogleContainerTools/skaffold/v2/proto/v2"
@@ -73,11 +69,13 @@ var (
 		Deployment      Type
 		StatefulSet     Type
 		ConfigConnector Type
+		CustomResource  Type
 	}{
 		StandalonePods:  "standalone-pods",
 		Deployment:      "deployment",
 		StatefulSet:     "statefulset",
 		ConfigConnector: "config-connector-resource",
+		CustomResource:  "custom-resource",
 	}
 )
 
@@ -215,41 +213,10 @@ func (r *Resource) checkRolloutStatus(ctx context.Context, cfg kubectl.Config) *
 		return &proto.ActionableErr{ErrCode: proto.StatusCode_STATUSCHECK_USER_CANCELLED}
 	}
 	details := r.cleanupStatus(string(b))
-	if err != nil {
-		return parseKubectlRolloutError(details, r.deadline, r.tolerateFailures, err)
-	}
 
-	client, cErr := kubernetesclient.Client(cfg.GetKubeConfig())
-	if cErr != nil {
-		log.Entry(ctx).Debugf("error attempting to create kubernetes client for k8s event listing: %s", err)
-	} else {
-		err = checkK8sEventsForPodFailedCreateEvent(ctx, client, r.namespace, r.name)
-	}
-
-	// additional logic added here which checks kubernetes events to see if skaffold managed pod has a FailedCreatEvent
-	// this can be raised by an admission controller and if we don't error here, skaffold will wait for the pod to come up
-	// indefinitely even thought the admission controller has denied it
 	return parseKubectlRolloutError(details, r.deadline, r.tolerateFailures, err)
 }
 
-func checkK8sEventsForPodFailedCreateEvent(ctx context.Context, client kubernetes.Interface, namespace string, deploymentName string) error {
-	// Create a watcher for events
-	eventList, err := client.CoreV1().Events(namespace).List(ctx, metav1.ListOptions{})
-	if err != nil {
-		return fmt.Errorf("error attempting to list kubernetes events in namespace: %s, %w", namespace, err)
-	}
-
-	for _, event := range eventList.Items {
-		if event.Reason == "FailedCreate" {
-			if strings.HasPrefix(event.InvolvedObject.Name, deploymentName+"-") {
-				errMsg := fmt.Sprintf("Failed to create Pod for Deployment %s: %s\n", deploymentName, event.Message)
-				return fmt.Errorf(errMsg)
-			}
-		}
-	}
-	return nil
-}
-
 func (r *Resource) CheckStatus(ctx context.Context, cfg kubectl.Config) {
 	var ae *proto.ActionableErr
 	switch r.rType {
@@ -257,6 +224,8 @@ func (r *Resource) CheckStatus(ctx context.Context, cfg kubectl.Config) {
 		ae = r.checkStandalonePodsStatus(ctx, cfg)
 	case ResourceTypes.ConfigConnector:
 		ae = r.checkConfigConnectorStatus()
+	case ResourceTypes.CustomResource:
+		ae = r.checkCustomResourceStatus()
 	default:
 		ae = r.checkRolloutStatus(ctx, cfg)
 	}
@@ -277,6 +246,13 @@ func (r *Resource) CheckStatus(ctx context.Context, cfg kubectl.Config) {
 		return
 	}
 	if err := r.fetchPods(ctx); err != nil {
+		// This hack is used to fail status check if deployment request is denied by admission webhook, without this status check
+		// will hang. This hack doesn't honor tolerate-failure-until-deadline flag, as when running kubectl apply with a standalone pod resource
+		// the command will fail directly, we won't even be here. So it probably makes more sense that we just stop here instead of repeating status checks.
+		if strings.Contains(err.Error(), validator.ReplicaFailureAdmissionErr) {
+			r.UpdateStatus(&proto.ActionableErr{ErrCode: proto.StatusCode_STATUSCHECK_UNKNOWN, Message: fmt.Sprintf("replica set creation failed: %s", err.Error())})
+			return
+		}
 		log.Entry(ctx).Debugf("pod statuses could not be fetched this time due to %s", err)
 	}
 }
@@ -446,6 +422,7 @@ func isErrAndNotRetryAble(statusCode proto.StatusCode) bool {
 		statusCode != proto.StatusCode_STATUSCHECK_DEPLOYMENT_ROLLOUT_PENDING &&
 		statusCode != proto.StatusCode_STATUSCHECK_STANDALONE_PODS_PENDING &&
 		statusCode != proto.StatusCode_STATUSCHECK_CONFIG_CONNECTOR_IN_PROGRESS &&
+		statusCode != proto.StatusCode_STATUSCHECK_CUSTOM_RESOURCE_IN_PROGRESS &&
 		statusCode != proto.StatusCode_STATUSCHECK_NODE_UNSCHEDULABLE &&
 		statusCode != proto.StatusCode_STATUSCHECK_UNKNOWN_UNSCHEDULABLE
 }
@@ -519,3 +496,31 @@ func (r *Resource) WithPodStatuses(scs []proto.StatusCode) *Resource {
 	}
 	return r
 }
+
+func (r *Resource) checkCustomResourceStatus() *proto.ActionableErr {
+	if len(r.resources) == 0 {
+		return &proto.ActionableErr{ErrCode: proto.StatusCode_STATUSCHECK_CUSTOM_RESOURCE_IN_PROGRESS}
+	}
+	var pendingResources []string
+	for _, resource := range r.resources {
+		ae := resource.ActionableError()
+		if ae == nil {
+			continue
+		}
+		switch ae.ErrCode {
+		case proto.StatusCode_STATUSCHECK_CUSTOM_RESOURCE_FAILED, proto.StatusCode_STATUSCHECK_CUSTOM_RESOURCE_TERMINATING:
+			return ae
+		case proto.StatusCode_STATUSCHECK_SUCCESS:
+			continue
+		default:
+			pendingResources = append(pendingResources, resource.Name())
+		}
+	}
+	if len(pendingResources) > 0 {
+		return &proto.ActionableErr{
+			ErrCode: proto.StatusCode_STATUSCHECK_CUSTOM_RESOURCE_IN_PROGRESS,
+			Message: fmt.Sprintf("custom resources not ready: %v", pendingResources),
+		}
+	}
+	return &proto.ActionableErr{ErrCode: proto.StatusCode_STATUSCHECK_SUCCESS}
+}
